/*
pg2_interface.c
Mario Amado Alves
2000-01-18
*/

#define RESULTS_MAX 500
#define NEW_RESULT RESULTS_MAX

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "/usr/include/postgresql/libpq-fe.h"

/* MAIN FUNCTIONS */
void Connect(char* Parameters);
void Disconnect(void);
int Submit(char* Command, int Result);
char* Get(int Result, int Tuple, int Field);
void Forget(int Result);

/* AUXILIARY FUNCTIONS */
int Last_OK(void);
char* Error_Message(void);
char* Host_Info(void);
char* Port_Info(void);
char* Options_Info(void);
char* TTY_Info(void);
char* Database_Info(void);
int OK(int Result);
int Tuples(int Result);
int Tuples_Qt(int Result);
int Fields_Qt(int Result);
char* Field_Name(int Result, int Field);
int Next_Free_Result(void);

/* GLOBALS */
PGconn* The_Connection;
PGresult* Results[RESULTS_MAX];
int Okay = 0;
char* Error_Msg = "No messages (Mario).";


/* MAIN FUNCTIONS */

void Connect(char* Parameters){
  clock_t time_1, time_2;
  time_1 = clock();
  The_Connection = PQconnectdb(Parameters);
  time_2 = clock();
  /* 
  printf("[Pg connect time = %E]", (time_2 - time_1) / CLOCKS_PER_SEC);
  */
  if(PQstatus(The_Connection) == CONNECTION_OK){
    Okay = 1;
    }
  else {
    Error_Msg = PQerrorMessage(The_Connection);
    PQfinish(The_Connection);
    The_Connection = NULL;
    Okay = 0;
    }
  /* printf("Error_Msg: %s\n", Error_Msg); */
  }

void Disconnect(void){
  if(The_Connection != NULL){
    PQfinish(The_Connection);
    The_Connection = NULL;
    }
  else {
    Error_Msg = "Invalid connection (Mario).";
    Okay = 0;
    }
  }

int Submit(char* Command, int Result){
  clock_t time_1, time_2;
  if(The_Connection != NULL
  && PQstatus(The_Connection) == CONNECTION_OK){
    if(Result == NEW_RESULT){
      Result = Next_Free_Result();
      }
    if(Result >= 0 && Result < RESULTS_MAX){
      if(Results[Result]){
        PQclear(Results[Result]);
        Results[Result] = NULL;
        }
      time_1 = clock();
      Results[Result] = PQexec(The_Connection, Command);
      time_2 = clock();
      /*
      printf(
        "[Pg submit time = %E]", (time_2 - time_1) / CLOCKS_PER_SEC );
      */
      if(PQresultStatus(Results[Result]) == PGRES_COMMAND_OK
      || PQresultStatus(Results[Result]) == PGRES_TUPLES_OK){
        Error_Msg = "";
        Okay = 1;
        }
      else {
        Error_Msg = "Invalid query or result (Mario).";
        Okay = 0;
        }
      }
    else {
      Error_Msg = "Invalid result index, or too many results (Mario).";
      Okay = 0;
      }
    }
  else {
    Error_Msg = "Invalid connection (Mario).";
    Okay = 0;
    }
  return(Result);
  }

char* Get(int Result, int Tuple, int Field){
  static char* Data = NULL;
  int Data_Size;
  clock_t time_1, time_2;
  time_1 = clock();
  if(Result >= 0 && Result < RESULTS_MAX && Results[Result]
  && PQresultStatus(Results[Result]) == PGRES_TUPLES_OK){
    Data_Size = PQgetlength(Results[Result], Tuple, Field);
    Data = (char*)realloc(Data, Data_Size + 1);
    if(Data){
      memcpy(Data, PQgetvalue(Results[Result], Tuple, Field), Data_Size);
      Data[Data_Size] = 0;
      Okay = 1;
      }
    else {
      Error_Msg = "Not enough memory (Mario).";
      Okay = 0;
      }
    }
  else {
    Error_Msg = "Invalid result "
      "(index out of range, null pointer, or no tuples) (Mario).";
    Okay = 0;
    }
  time_2 = clock();
  /*
  printf("[Pg get time = %E]", (time_2 - time_1) / CLOCKS_PER_SEC);
  */
  return(Data);
  }

void Forget(int Result){
  if(Result >= 0 && Result < RESULTS_MAX){
    if(Results[Result]){
      PQclear(Results[Result]);
      }
    Results[Result] = NULL;
    }
  }


/* RESULT OF LAST OPERATION */

int Last_OK(void){
  return Okay;
  }

char* Error_Message(void){
  return(Last_OK() == 0 ? Error_Msg : "No errors (Mario).");
  }


/* CONNECTION INFO */

char* Host_Info(void){
  return(PQhost(The_Connection));
  }
char* Port_Info(void){
  return(PQport(The_Connection));
  }
char* Options_Info(void){
  return(PQoptions(The_Connection));
  }
char* TTY_Info(void){
  return(PQtty(The_Connection));
  }
char* Database_Info(void){
  return(PQdb(The_Connection));
  }


/* COMMAND RESULT INFO */

int OK(int Result){
  int ok;
  if(Results[Result] && (ok=PQresultStatus(Results[Result]))){
    return(ok == PGRES_COMMAND_OK || ok == PGRES_TUPLES_OK ? 1 : 0);
    }
  else {
    return(0);
    }
  }
int Tuples(int Result){
  return(PQresultStatus(Results[Result]) == PGRES_TUPLES_OK ? 1 : 0);
  }
int Tuples_Qt(int Result){
  return(PQntuples(Results[Result]));
  }
int Fields_Qt(int Result){
  return(PQnfields(Results[Result]));
  }
char* Field_Name(int Result, int Field){
  return(PQfname(Results[Result], Field));
  }


/* RESULT POOL */

int Next_Free_Result(void){
  static int First_Call = 1;
  int i;
  if(First_Call == 1){
    for(i = 0; i < RESULTS_MAX; i++) Results[i] = NULL;
    First_Call = 0;
    }
  for(i = 0; i < RESULTS_MAX; i++){
    if(Results[i] == NULL){
      break;
      }
    }
  return(i < RESULTS_MAX ? i : -1);
  }
